# Generated via
#  `rails generate hyrax:work Crc1280Experiment`
module Hyrax
  class Crc1280ExperimentPresenter < Hyrax::WorkShowPresenter
    include ComplexFilterHelper

    delegate :approval_number, :complex_date, :complex_funding_reference, :complex_identifier, :complex_person,
             :complex_relation, :complex_subject, :coverage, :crc_resource_type, :crc_work_type, :doi, :folder_name,
             :experiment_description, :extra_information, :modality, :parent_work_id, :software_version, :tombstone_status, :tombstone_date, :is_tombstoned, :proxy_depositor, to: :solr_document

    def list_all_children(options = {})
      all_children = {
        'experiment' => {},
        'subject' => {},
        'session' => {},
        'modality' => {}
      }
      experiment =  Hyrax.query_service.find_by(id: self.id)

      # subject filter query if filter available
      subject_query = build_query(subject_field_mappings, options[:filter])
      session_query = build_query(session_field_mappings, options[:filter])
      modality_query = build_query(modality_field_mappings, options[:filter])

      # get childrens
      complex_subjects = filtered_subjects(subject_query, session_query, modality_query)
      complex_sessions = filtered_sessions(subject_query, session_query, modality_query)
      complex_modalities = filtered_modalities(subject_query, session_query, modality_query)

      child_subjects = complex_subjects.sort_by_title(experiment.source.first)

      members = Kaminari.paginate_array(experiment.file_sets + child_subjects).page(options[:page] || 1).per(options[:per_pag])

      # # get children
      all_children['experiment'][id] = gather_children(members)

      child_subjects.each do |subject|
        # get children sessions
        subject_file_sets = (session_query.present? || modality_query.present?) ? [] : subject.file_sets
        child_sessions = complex_sessions.sort_by_title(subject.source_identifier)

        # get children
        all_children['subject'][subject.id] = gather_children(subject_file_sets + child_sessions)

        child_sessions.each do |session|
          session_file_sets = modality_query.present? ? [] : session.file_sets
          # modality filter query if filter available
          
          child_modalities = complex_modalities.sort_by_title(session.source_identifier)
          
          # get children
          all_children['session'][session.id] = gather_children(session_file_sets + child_modalities)

          #get modality children
          child_modalities.each do |modality|
            modality_members = modality.file_sets
            all_children['modality'][modality.id] = gather_children(modality_members)
          end
        end
      end
      all_children
    end

    private

    def gather_children(members)
      data = {'files' => [], 'children' => [], 'info' => {}}

      return {} unless members.any?

      members.each do |member|
        if member.class.name == "Hyrax::FileSet"
          # When the data has not been processed by the actors, 
          # the label property is missing and it throws http errors
          if member.title[0]
            data['info'][member.id] = {'title' => member.title[0], 'label' => member.title[0]}
            data['files'] << member.id
          end
        else
          if member.is_a?(ComplexSubject)
            data['info'][member.id] = {'title' => member.subject_title}
          elsif member.is_a?(ComplexSession)
            data['info'][member.id] = {'title' => member.session_title}
          elsif member.is_a?(ComplexModality)
            data['info'][member.id] = {'title' => member.modality_title || member.modality}
          end

          data['children'] << member.id
        end
      end
      
      data
    end

  end
end
